%Obtain initial scaling parameter: Get the scaling parameter such that the
%scaling residuals are balanced. Does NOT allows cell defined variables
%and uses the explicit dual variables to compute the residuals.
%%******************************************************************
%%
%%   theta = TEBD_GetInitialScaling_NC_v2_1(prob,x,u,sigma)
%%
%%  INPUT:              prob = (struct) problem definition
%%                         x = the initial point x (1st block variable)
%%                         u = the initial point u (2nd block variable)
%%                     sigma = sgima parameter of 2EBD-HPE
%%
%%
%%  OUTPUT:            theta = the initial value of the scaling parameter
%%
%%
%% [1] R. D.C Monteiro, C. Ortiz and B. F. Svaiter. A first-order
%%     block-decomposition method for solving two-easy-block structured
%%     semidefinite programs, 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2EBD-HPE:
%% Modified by C.Ortiz
%% Last Modified: 5/31/2013

function theta = TEBD_GetInitialScaling_NC_v2_1(prob,x,u,sigma)
theta = 1;
NotBalanced = 1;
iter = 0;
maxpower = 20;
direction = 0;
while NotBalanced && theta >= 2^-maxpower && theta <= 2^maxpower && iter <= maxpower
    [res1,res2] = GetFirstIterationResiduals(prob,x,u,theta,sigma);
    if direction ~= 2 && 2*res1 < res2
        direction = 1;
        theta = theta*2;
    elseif direction ~= 1 && 2*res2 < res1
        direction = 2;
        theta = theta/2;
    else
        NotBalanced = 0 ;
    end
    iter = iter + 1;
end

end

function [res1,res2] = GetFirstIterationResiduals(prob,x,u,theta,sigma)
lambda = sigma/sqrt(theta);
Z_tilde.pos=1;

% perform 1st prox
gradf_x = prob.gradf(x);
gradf_x_plus_u = gradf_x+u;
[x_tilde,Z_tilde] = prob.resolvent1(x - gradf_x_plus_u*(theta*lambda),Z_tilde,lambda*theta);

% perform 2nd prox using Moreau's identity
u_plus_lambda_x = (u +x_tilde*lambda);
[resolv2,y_tilde] = prob.resolvent2(u_plus_lambda_x/lambda,lambda);%recovers dual variable in y_tilde
u_tilde = u_plus_lambda_x - resolv2*lambda;

% compute v\in T(x) or v\in T^\epsilon(x)
v_1 = (((x-x_tilde)/lambda) +((u_tilde-u)*theta));
v_2 = ((u-u_tilde)/lambda);


%% Obtain the residuals relevant to each problem
obj = prob.evalf(x_tilde,y_tilde,Z_tilde);
residuals = prob.residuals(x_tilde,u_tilde,v_1/theta,v_2,obj,y_tilde,Z_tilde,u_plus_lambda_x,lambda);

res1 = residuals.pfeas;
res2 = residuals.dfeas;
end
