%Obtain initial scaling parameter: Get the scaling parameter such that the
%scaling residuals are balanced. Allows cell defined variables
%and uses the explicit dual variables to compute the residuals.
%%******************************************************************
%%
%%   theta = TEBD_GetInitialScaling_v2_1(prob,x,u,sigma)
%%
%%  INPUT:              prob = (struct) problem definition
%%                         x = the initial point x (1st block variable)
%%                         u = the initial point u (2nd block variable)
%%                     sigma = sgima parameter of 2EBD-HPE
%%
%%
%%  OUTPUT:            theta = the initial value of the scaling parameter
%%
%%
%% [1] R. D.C Monteiro, C. Ortiz and B. F. Svaiter. A first-order
%%     block-decomposition method for solving two-easy-block structured
%%     semidefinite programs, 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2EBD-HPE:
%% Modified by C.Ortiz
%% Last Modified: 5/31/2013

function theta = TEBD_GetInitialScaling_v2_1(prob,x,u,sigma)
theta = 1;
NotBalanced = 1;
iter = 0;
maxpower = 20;
direction = 0;
while NotBalanced && theta >= 2^-maxpower && theta <= 2^maxpower && iter <= maxpower
    [res1,res2] = GetFirstIterationResiduals(prob,x,u,theta,sigma);
    if direction ~= 2 && 2*res1 < res2
        direction = 1;
        theta = theta*2;
    elseif direction ~= 1 && 2*res2 < res1
        direction = 2;
        theta = theta/2;
    else
        NotBalanced = 0 ;
    end
    iter = iter + 1;
end

end

function [res1,res2] = GetFirstIterationResiduals(prob,x,u,theta,sigma)
lambda = sigma/sqrt(theta);
Z_tilde.pos = 1;

% perform 1st prox
gradf_x = prob.gradf(x);
[x_tilde,Z_tilde] = prob.resolvent1(TEBD_ops(x,'X-alpha*(Y+X2)',gradf_x,theta*lambda,u),Z_tilde,lambda*theta);

% perform 2nd prox using Moreau's identity
[u_plus_lambda_x_div_lambda, u_plus_lambda_x] = TEBD_ops(u ,'(X+Y*alpha)/alpha',x_tilde,lambda);
[resolv2,y_tilde] = prob.resolvent2(u_plus_lambda_x_div_lambda,lambda);%recovers dual variable in y_tilde
u_tilde = TEBD_ops(u_plus_lambda_x ,'-', resolv2,lambda);

% compute v\in T(x) or v\in T^\epsilon(x)
[~, v_1_div_theta] = TEBD_ops(x,'[(X-Y)/alpha +(X2-Y2)*alpha2]/alpha2-X3-Y3',x_tilde,lambda,u_tilde,u,theta,gradf_x,u_tilde);
[v_2] = TEBD_ops(u,'(X-Y)/alpha+X2',u_tilde,lambda,x_tilde);


%% Obtain the residuals relevant to each problem
obj = prob.evalf(x_tilde,y_tilde,Z_tilde);
residuals = prob.residuals(x_tilde,u_tilde,v_1_div_theta,v_2,obj,y_tilde,Z_tilde,u_plus_lambda_x,lambda);

res1 = residuals.pfeas;
res2 = residuals.dfeas;
end
