%TEBD v2.1: main solver. Generates the dual variables at every of the algorithm. 
%Does NOT allows cell defined variables
%%
%%
%% Solves two-easy-block structured convex optimization problems of the form:
%%
%%                              min f(x)+h_1(x)+h_2(x)
%% results = TEBD_NC_v2_1(par,prob)
%%
%% input:
%%		- par                : (struct) parameters of the algorithm
%%		- prob               : (struct) problem definition
%%
%% output:
%%		- results            : (struct) contains the results after running 2EBD-HPE using the given parameters (par) on the given problem (prob)
%%
%% [1] R. D.C Monteiro, C. Ortiz and B. F. Svaiter. A first-order
%%     block-decomposition method for solving two-easy-block structured
%%     semidefinite programs, 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2EBD-HPE:
%% Modified by C.Ortiz
%% Last Modified: 5/31/2013
function results = TEBD_NC_v2_1(par,prob)


sigma   = par.sigma;



x = prob.x0;
u = prob.u0;



par.AlgName = '2EBD-v2.1-NoCell';
fprintf('\n\n ****Begin 2EBD V2.1. iterations = %d ****\n', par.maxit);
fprintf(par.AlgName);
fprintf('\n');
fprintf(TEBD_LegendAppend(par));
fprintf('\n');

tstart = tic;
%%-----------------------------------------




if isfield(par,'balancedinit') && par.balancedinit >0
    theta = TEBD_GetInitialScaling_NC_v2_1(prob,x,u,sigma);
else
    theta = 1;
end

lambda = sigma/sqrt(theta);


%%
%% dynamicscaling = 1: dynamic scaling of the primal inner product (dynamic change of the weight in primal and dual intermediate stepsizes)
%                   0: no dynamic scaling of the primal inner product
dynamicscaling = par.dynamicscaling;

%dynamic scaling flag
isdynamicscaling = 0;

if dynamicscaling == 1
    %% dyn_scale_update_it: # of iterations to update the primal inner product
    %                       scaling
    dyn_scale_update_it = par.dyn_scale_updateiteration;
    
    %% scaleratio: accepted ratio between primal and dual relative residuals
    %              for of the dynamic inner product scaling
    scaleratio = par.scaleratio;
    
    %% scalecorrection: change made in the primal inner product if the primal
    %                   and dual relative residuals are not within the accepted
    %                   ratio for the dynamic inner product scaling
    scalecorrection = par.scalecorrection;
end

%% print headers
TEBD_PrintHeaders;

%% save time preprocessing
runhist.preprocesstime = toc(tstart);


stopstatus = 1;

tstart = tic;

Z_tilde.pos = 1;

%%
for iter = 1:par.maxit;
    %% Perform main iteration
    
    % perform 1st prox
    gradf_x = prob.gradf(x);
    gradf_x_plus_u = gradf_x+u;
    [x_tilde,Z_tilde] = prob.resolvent1(x - gradf_x_plus_u*(theta*lambda),Z_tilde,lambda*theta);
    
    % perform 2nd prox using Moreau's identity
    u_plus_lambda_x = u +x_tilde*lambda;
    [resolv2,y_tilde] = prob.resolvent2(u_plus_lambda_x/lambda,lambda);%recovers dual variable in y_tilde
    u_tilde = u_plus_lambda_x - resolv2*lambda;
    
    % compute v\in T(x) or v\in T^\epsilon(x)
    v_1 = ((x-x_tilde)/lambda) + ((u_tilde-u)*theta);
    v_2 = (u-u_tilde)/lambda;
    
    if par.adaptive == 0
        lambda_a = lambda;
    else
        %% find adaptive (aggresive) stepsize lambda
        [lambda_a] = GetAdaptiveLambdaScaled(prob,lambda,sigma,x,x_tilde,v_1,u,u_tilde,v_2,gradf_x,theta);
    end
    
    % perform the extragradient step
    x = x - v_1*lambda_a;
    u = u - v_2*lambda_a;
    
    %% save the time until this iteration
    secs = toc(tstart);
    
    %% Obtain the residuals relevant to each problem
    obj = prob.evalf(x_tilde,y_tilde,Z_tilde);
    residuals = prob.residuals(x_tilde,u_tilde,v_1/theta,v_2,obj,y_tilde,Z_tilde,u_plus_lambda_x,lambda);
    
    %% Save History: save all the information to evaluate the performance
    runhist = TEBD_SaveHistory(obj,runhist,iter,residuals,secs,lambda_a/lambda,theta);
    
    %% check all stopping conditions
    [stopstatus,printstatus,statusstr,runhist] = TEBD_Check_StopCrit(par,residuals,runhist,iter,stopstatus);
    if printstatus
        TEBD_PrintCurrentIterate(par,runhist,iter,theta,residuals);
        fprintf(statusstr);
    end
    
    if residuals.resnorm < par.tol || iter >= par.maxit
        
        %% print last iterate
        TEBD_PrintCurrentIterate(par,runhist,iter,theta,residuals);
        fprintf(statusstr);
        
        %% Save results
        results = TEBD_GetFinalValues_1(runhist,iter,par,prob,x,u,u_tilde,x_tilde,y_tilde,Z_tilde,secs);
        break
    end
    
    %% print current progress
    if  iter == 1 || mod(iter,par.printit)==0
        
        %% print  iterate
        TEBD_PrintCurrentIterate(par,runhist,iter,theta,residuals);
    end
    
    %% Dynamic scaling of the inner product
    if dynamicscaling == 1 && (max(residuals.res1,residuals.res2) > par.tol2||(residuals.res1>residuals.res2*5)||(residuals.res2>residuals.res1*5))
        if isdynamicscaling == 0
            fprintf(' ****Starting dynamic scaling at iteration = %d ****\n', iter);
            isdynamicscaling = 1;
        end
        if mod(iter,dyn_scale_update_it)==0
            wrel_norm2 = wrel_norm2 * residuals.res2^(1/dyn_scale_update_it);
            wrel_norm1 = wrel_norm1 * residuals.res1^(1/dyn_scale_update_it);
            if  wrel_norm2/wrel_norm1 > scaleratio
                theta = theta/(scalecorrection^2);
                TEBD_PrintCurrentIterate(par,runhist,iter,theta,residuals);
            elseif  wrel_norm1/wrel_norm2 > scaleratio
                theta = theta*(scalecorrection^2);
                TEBD_PrintCurrentIterate(par,runhist,iter,theta,residuals);
            end
            lambda = sigma/sqrt(theta);
        elseif mod(iter,dyn_scale_update_it) == 1
            % Short memory agregation of the decition residuals
            wrel_norm2 =  residuals.res2^(1/dyn_scale_update_it);
            wrel_norm1 = residuals.res1^(1/dyn_scale_update_it);
        else
            % Short memory agregation of the decition residuals
            wrel_norm2 = wrel_norm2 * residuals.res2^(1/dyn_scale_update_it);
            wrel_norm1 = wrel_norm1 * residuals.res1^(1/dyn_scale_update_it);
        end
    elseif isdynamicscaling == 1 && dynamicscaling == 1
        fprintf(' ****Stopping dynamic scaling at iteration = %d ****\n', iter);
        isdynamicscaling = 0;
    end
    
end

fprintf('\n\n ****End 2EBD-HPE algorithm ****\n');
%%---------------------------------------------------------------
%% end of main loop
%%---------------------------------------------------------------
%%
%% *****************************************************************************
end

function [lambda_a] = GetAdaptiveLambdaScaled(prob,lambda,sigma,x,x_tilde,v_1,u,u_tilde,v_2,gradf_x,theta)
%Compute the error
eps = prob.error1(x,x_tilde,gradf_x);

lambda_sq = lambda^2;

%Compute the residual v
v_x = v_1;
v_u = v_2;
Norm_v_x_sq = norm(v_x)^2;
Norm_v_u_sq = norm(v_u)^2;


Delta_x = (x_tilde-x);
Delta_u = (u_tilde-u);
Norm_Delta_x = norm(Delta_x);
Norm_Delta_u = norm(Delta_u);
Norm_Delta_x_sq = Norm_Delta_x^2;
Norm_Delta_u_sq = Norm_Delta_u^2;

tr_v_Delta_x = v_x'*Delta_x;
tr_v_Delta_u = v_u'*Delta_u;

alpha = sigma^2*(Norm_Delta_x_sq/theta+Norm_Delta_u_sq);

a = (Norm_v_x_sq)/theta + Norm_v_u_sq;
b = (lambda*Norm_v_x_sq+tr_v_Delta_x)/theta + lambda*Norm_v_u_sq + tr_v_Delta_u + eps;
c = (lambda_sq*Norm_v_x_sq+2*lambda*tr_v_Delta_x+Norm_Delta_x_sq)/theta+lambda_sq*Norm_v_u_sq+Norm_Delta_u_sq+2*lambda*(tr_v_Delta_u+eps)-alpha;
discriminant = sqrt(b^2-a*c);
if b <= 0
    beta = (-b+discriminant)/a;
else
    beta = -c/(b+discriminant);
end
if beta <=  1e-13
    beta = 0;
else
    %fprintf('%d beta = %8.3e\n',iter,beta);
end
lambda_a = lambda + beta;
end
