%Get Frequency Assignment Problem (FAP) two-easy-block structure definition
%with the stopping criterion residual set to max{pfeas,dfeas} or max{pfeas,dfeas,gap}
%%******************************************************************
%% TEBD_GetProblem_FAP:
%%   defines the necesary subroutines to solve 
%%                                 
%%                              min f(x)+h_1(x)+h_2(x)
%%
%%   [prob,par] = TEBD_GetProblem_FAP(prob_sdp,par)
%%
%%  INPUT:     prob_sdp = (struct) problem definition for obtaining the SDP
%%             par      = (struct) parameters of the algorithm to update
%%
%%  OUTPUT:             prob = (struct) problem definition
%%           prob.resolvent1 = solves the resolvent of h_1: 
%%                             (I+\lambda\partial h_1)^-1
%%           prob.resolvent2 = solves the resolvent of h_2: 
%%                             (I+\lambda\partial h_2)^-1
%%                prob.trace = computes the innerproduct between x and x_2: <x,x_2>
%%                prob.evalf = evaluates the value of f at x: f(x)
%%                prob.gradf = evaluates the gradient of f at x: \nabla f(x)
%%               prob.error1 = for linear f(x) this error is zero
%%                             computes the error of the first block:
%%                             \epsilon'_k in Lemma 4.3 in [1]
%%                prob.normb = evaluates the norm of b: \|b\|;
%%                 prob.norm = evaluates the norm of a point x: \|x\|;
%%                prob.normC = evaluates the norm of C: \|C\|;
%%             prob.resduals = (struct) contains the specific residuals of 
%%                                      the problem at the current iteration 
%%            prob.resduals2 = (struct) contains the specific residuals of 
%%                                      the problem at the current iteration 
%%                                      without using the explicit dual variables
%%                       par = (struct) parameters of the algorithm to update 
%%           
%% [1] R. D.C Monteiro, C. Ortiz and B. F. Svaiter. A first-order 
%%     block-decomposition method for solving two-easy-block structured 
%%     semidefinite programs, 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2EBD-HPE:
%% Modified by C.Ortiz
%% Last Modified: 5/31/2013
%% 
function [prob,par] = TEBD_GetProblem_FAP(prob_sdp,par)


prob_sdp = TEBD_GetFAP_SDP(prob_sdp);
prob = TEBD_GetFAP(prob_sdp);

prob.scalewithgap = 1;
prob.reswithgap = 0;

par.scalewithgap = prob.scalewithgap;
par.reswithgap = prob.reswithgap;

%% Set the required operations
prob.x0(prob.E1) = 1;% 
prob.u0 = prob.x0;

prob.resolvent1 = @(x,Z,lambda)TEBD_GetProjectionToSemidefiniteCone_v2_2(prob.n,x,Z,lambda);
prob.resolvent2 = @(u,lambda)TEBD_GetProjectionToK_FAP_v2(prob,u,lambda);
prob.trace = @(x,x2)x'*x2;
prob.bmat = zeros(size(prob.C,1),1);
prob.bmat(prob.E1)=1;
prob.bmat(prob.E2)=-sqrt(2)/(prob.k-1);
prob.bmat(prob.E3)=sqrt(2)/(prob.k-1);
prob.evalf = @(x,y,Z)TEBD_EvalObj_FAP_v2(prob,x,y);
prob.evalf2 = @(x,u,v,Bs)TEBD_EvalObj1(prob,x,u,v);
prob.gradf = @(x)prob.C;
prob.error1 = @(x,x_tilde,grad_f)0;
prob.normb = sqrt(prob.n+(prob.m-prob.n)*4/((prob.k-1)*(prob.k-1)));
prob.norm = @(x)norm(x);
prob.normC = norm(prob.C);
prob.normC2 = prob.normC*2;
prob.residuals = @(x,u,N_F_x,N_F_u,obj,y,Z,lambda_yhat,lambda)GetResiduals(prob,x,u,N_F_x,N_F_u,obj,y,Z,lambda_yhat,lambda);
prob.residuals2 = @(x,u,N_F_x,N_F_u,obj)GetResiduals2(prob,x,u,N_F_x,N_F_u,obj);
end

function residuals = GetResiduals2(prob,x,~,F_x,~,obj)

Norm_Ax_minus_b_ORIG_1_sq = norm(x(prob.E1) - 1)^2;
Norm_Ax_minus_b_ORIG_2_sq = 2*norm(min(0,x(prob.E2) + sqrt(2)/(prob.k-1)))^2 + 2*norm((x(prob.E3) + sqrt(2)/(prob.k-1)))^2;
Norm_Ax_minus_b_ORIG = sqrt(Norm_Ax_minus_b_ORIG_1_sq+Norm_Ax_minus_b_ORIG_2_sq);
Norm_Ax_minus_b2 = sqrt(Norm_Ax_minus_b_ORIG_1_sq+Norm_Ax_minus_b_ORIG_2_sq/2);

N_F_x = norm(F_x); 

residuals = TEBD_GetResiduals(prob,obj,Norm_Ax_minus_b_ORIG,N_F_x,Norm_Ax_minus_b2);
end

function residuals = GetResiduals(prob,x,~,~,~,obj,ys,Zs,~,~)

Norm_Ax_minus_b_ORIG_1_sq = norm(x(prob.E1) - 1)^2;
Norm_Ax_minus_b_ORIG_2_sq = norm(min(0,sqrt(2)*x(prob.E2) + 2/(prob.k-1)))^2 + norm((sqrt(2)*x(prob.E3) + 2/(prob.k-1)))^2;
Norm_Ax_minus_b_ORIG = sqrt(Norm_Ax_minus_b_ORIG_1_sq+Norm_Ax_minus_b_ORIG_2_sq);
Norm_Ax_minus_b2 = sqrt(Norm_Ax_minus_b_ORIG_1_sq+Norm_Ax_minus_b_ORIG_2_sq/2);

N_F_x = norm(prob.C-ys.Aty-Zs.Z);%N_F_x = prob.norm(prob.C-ys.Aty-Zs.Z);

residuals = TEBD_GetResiduals(prob,obj,Norm_Ax_minus_b_ORIG,N_F_x,Norm_Ax_minus_b2);
end
