%Get a standard form SDP with a two-easy-block structure definition
%%******************************************************************
%% TEBD_GetProblem_SDP_StandardForm:
%%   defines the necesary subroutines to solve
%%
%%                              min f(x)+h_1(x)+h_2(x)
%%
%%   prob = TEBD_GetProblem_SDP_StandardForm(prob_sdp)
%%
%%  INPUT:     prob = (struct) problem definition in standard for SDP
%%                                 using sparse SDPT3 data structure.
%%
%%  OUTPUT:             prob = (struct) problem definition
%%           prob.resolvent1 = solves the resolvent of h_1:
%%                             (I+\lambda\partial h_1)^-1
%%           prob.resolvent2 = solves the resolvent of h_2:
%%                             (I+\lambda\partial h_2)^-1
%%                prob.trace = computes the innerproduct between x and x_2: <x,x_2>
%%                prob.evalf = evaluates the value of f at x: f(x)
%%                prob.gradf = evaluates the gradient of f at x: \nabla f(x)
%%               prob.error1 = for linear f(x) this error is zero
%%                             computes the error of the first block:
%%                             \epsilon'_k in Lemma 4.3 in [1]
%%                prob.normb = evaluates the norm of b: \|b\|;
%%                 prob.norm = evaluates the norm of a point x: \|x\|;
%%                prob.normC = evaluates the norm of C: \|C\|;
%%             prob.resduals = (struct) contains the specific residuals of 
%%                                      the problem at the current iteration 
%%             prob.resduals2 = (struct) contains the specific residuals of 
%%                                      the problem at the current iteration 
%%                                      without using the explicit dual variables
%%
%% [1] R. D.C Monteiro, C. Ortiz and B. F. Svaiter. A first-order
%%     block-decomposition method for solving two-easy-block structured
%%     semidefinite programs, 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2EBD-HPE:
%% Modified by C.Ortiz
%% Last Modified: 5/31/2013
%%
function [prob,par] = TEBD_GetProblem_SDP_StandardForm(prob,par)


prob.scalewithgap = 1; 
prob.reswithgap = 0;

par.scalewithgap = prob.scalewithgap;
par.reswithgap = prob.reswithgap;

prob.Chol = TEBD_cholAAt(prob.blk,prob.At,length(prob.b));
for p = 1:size(prob.blk,1)
    prob.C{p} = full(prob.C{p});
end



%% Set the required operations
prob.x0 = TEBD_ops(prob.C,'zeros');
prob.u0 = TEBD_ops(prob.C,'zeros');

prob.resolvent1 = @(x,Z,lambda)TEBD_GetMatProjectionToGeneralCone_v2_2(prob.blk,x,Z,lambda);
prob.resolvent2 = @(u,lambda)ProjectToConstraints(prob,u,lambda);
prob.trace = @(x,x2)blktrace(prob.blk,x,x2);
prob.evalf = @(x,y,Z)TEBD_EvalObj_SDP_v2(prob,x,y);
prob.evalf2 = @(x,u,v,Bs)TEBD_EvalObj1(prob,x,u,v);
prob.gradf = @(x)prob.C;
prob.error1 = @(x,x_tilde,grad_f)0;
prob.normb = norm(prob.b);
prob.normC2 = prob.normC*2;
prob.norm = @(x)TEBD_ops(x,'norm');
prob.residuals = @(x,u,N_F_x,N_F_u,obj,y,Z,lambda_yhat,lambda)GetResiduals(prob,x,u,N_F_x,N_F_u,obj,y,Z,lambda_yhat,lambda);
prob.residuals2 = @(x,u,N_F_x,N_F_u,obj)GetResiduals2(prob,x,u,N_F_x,N_F_u,obj);
end


function [proju,y] = ProjectToConstraints(prob,u,lambda)
b_minus_Au = prob.b - TEBD_AXfun(prob.blk,prob.At,u);
y.y = prob.Chol.S*(prob.Chol.R\(prob.Chol.Rt\(prob.Chol.St*b_minus_Au)));
y.Aty = TEBD_Atyfun(prob.blk,prob.At,y.y);
proju = TEBD_ops(u,'+',y.Aty);
if nargout > 1
    y.y = TEBD_ops(y.y,'*',lambda);
    y.Aty = TEBD_ops(y.Aty,'*',lambda);
end
end

function residuals = GetResiduals(prob,x,~,~,~,obj,ys,Zs,~,~)
Ax_minus_b = TEBD_AXfun(prob.blk,prob.At,x)-prob.b;

Norm_Ax_minus_b_ORIG =  norm(Ax_minus_b);

Norm_Ax_minus_b2 = norm(prob.Chol.S*(prob.Chol.R\(prob.Chol.Rt\(prob.Chol.St*Ax_minus_b))));

N_F_x = prob.norm(TEBD_ops(prob.C,'X-Y-alpha',ys.Aty,Zs.Z));

residuals = TEBD_GetResiduals(prob,obj,Norm_Ax_minus_b_ORIG,N_F_x,Norm_Ax_minus_b2);
end

function residuals = GetResiduals2(prob,x,~,F_x,~,obj,~,~,~,~)
Ax_minus_b = TEBD_AXfun(prob.blk,prob.At,x)-prob.b;

Norm_Ax_minus_b_ORIG =  norm(Ax_minus_b);

Norm_Ax_minus_b2 = norm(prob.Chol.S*(prob.Chol.R\(prob.Chol.Rt\(prob.Chol.St*Ax_minus_b))));

N_F_x = prob.norm(F_x);

residuals = TEBD_GetResiduals(prob,obj,Norm_Ax_minus_b_ORIG,N_F_x,Norm_Ax_minus_b2);
end

