%Get the problem for an exact/inexact ell_1 minimization problem
%with a two-easy-block structure definition
%%******************************************************************
%% TEBD_GetProblem_ell_1:
%%   defines the necesary subroutines to solve
%%
%%                              min f(x)+h_1(x)+h_2(x)
%%
%%   prob = TEBD_GetProblem_ell_1(prob_sdp)
%%
%%  INPUT:     prob = (struct) problem definition in standard for SDP
%%                                 using sparse SDPT3 data structure.
%%
%%  OUTPUT:             prob = (struct) problem definition
%%           prob.resolvent1 = solves the resolvent of h_1:
%%                             (I+\lambda\partial h_1)^-1
%%           prob.resolvent2 = solves the resolvent of h_2:
%%                             (I+\lambda\partial h_2)^-1
%%                prob.trace = computes the innerproduct between x and x_2: <x,x_2>
%%                prob.evalf = evaluates the value of f at x: f(x)
%%                prob.gradf = evaluates the gradient of f at x: \nabla f(x)
%%               prob.error1 = for linear f(x) this error is zero
%%                             computes the error of the first block:
%%                             \epsilon'_k in Lemma 4.3 in [1]
%%                prob.normb = evaluates the norm of b: \|b\|;
%%                 prob.norm = evaluates the norm of a point x: \|x\|;
%%                prob.normC = evaluates the norm of C: \|C\|;
%%             prob.resduals = (struct) contains the specific residuals of
%%                                      the problem at the current iteration
%%             prob.resduals2 = (struct) contains the specific residuals of
%%                                      the problem at the current iteration
%%                                      without using the explicit dual variables
%%
%% [1] R. D.C Monteiro, C. Ortiz and B. F. Svaiter. A first-order
%%     block-decomposition method for solving two-easy-block structured
%%     semidefinite programs, 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2EBD-HPE:
%% Modified by C.Ortiz
%% Last Modified: 5/31/2013
%%
function [prob,par] = TEBD_GetProblem_ell_1(prob,par)


prob.scalewithgap = 1;
prob.reswithgap = 0;

par.scalewithgap = prob.scalewithgap;
par.reswithgap = prob.reswithgap;

prob.Chol = TEBD_cholAAt({'l',prob.m},{prob.A'},prob.m);




%% Set the required operations

zero0 = zeros(prob.n,1);

prob.resolvent1 = @(x,Z,lambda)GetResolvent_ell_1_norm(x,lambda);
if strcmp(prob.format,'RAND_ell_1_inexact')
    prob.resolvent2 = @(u,lambda)ProjectToConstraintsInexact(prob,u,lambda);
else
    prob.resolvent2 = @(u,lambda)ProjectToConstraints(prob,u,lambda);
end
prob.trace = @(x,x2)blktrace(prob.blk,x,x2);
prob.evalf = @(x,y,Z)EvalObj_ell_1(prob,x);
prob.evalf2 = @(x,u,v,Bs)EvalObj_ell_1(prob,x);
prob.gradf = @(x)zero0;
prob.error1 = @(x,x_tilde,grad_f)0;
prob.normb = norm(prob.b);
prob.normC = 1;
prob.normC2 = 1;
prob.norm = @(x)TEBD_ops(x,'norm');
prob.residuals = @(x,u,N_F_x,N_F_u,obj,y,Z,lambda_yhat,lambda)GetResiduals(prob,x,u,N_F_x,N_F_u,obj,y,Z,lambda_yhat,lambda);
prob.residuals2 = @(x,u,N_F_x,N_F_u,obj)GetResiduals2(prob,x,u,N_F_x,N_F_u,obj);
end


function [xres,Z] = GetResolvent_ell_1_norm(x_0,lambda)
xres = x_0 - min(max(x_0,-lambda),lambda);
Z = 0;
end


function [proju,y] = ProjectToConstraintsInexact(prob,u,lambda)
Au_minus_b =  prob.A*u -  prob.b;
y.y = prob.Chol.S*(prob.Chol.R\(prob.Chol.Rt\(prob.Chol.St*(ProjectToBall(prob,Au_minus_b)-Au_minus_b))));
y.Aty = prob.A'*y.y;
proju = u+y.Aty;
if nargout > 1
    y.y = (y.y*lambda);
    y.Aty = y.Aty*lambda;
end
end

function projx =  ProjectToBall(prob,y)
    projx = y*min(prob.epsilon/norm(y),1);
end

function [proju,y] = ProjectToConstraints(prob,u,lambda)
b_minus_Au = prob.b - prob.A*u;
y.y = prob.Chol.S*(prob.Chol.R\(prob.Chol.Rt\(prob.Chol.St*b_minus_Au)));
y.Aty = prob.A'*y.y;
proju = u+y.Aty;
if nargout > 1
    y.y = (y.y*lambda);
    y.Aty = y.Aty*lambda;
end
end

function obj = EvalObj_ell_1(prob,x)
%Compare with the signal we want to recover.
obj(1) = norm(x-prob.x_star,1); 
%Can also check the sparsity recovered 
% obj(1) = sum((prob.x_star~=0&x==0)|(prob.x_star==0&x~=0)); 
obj(2) = norm(prob.A*x-prob.b);
end

function residuals = GetResiduals(prob,x,~,F_x,F_u,obj,ys,Zs,~,~)
Ax_minus_b = prob.A*F_u;

Norm_Ax_minus_b_ORIG =  norm(Ax_minus_b);

Norm_Ax_minus_b2 = norm(prob.Chol.S*(prob.Chol.R\(prob.Chol.Rt\(prob.Chol.St*Ax_minus_b))));

N_F_x = prob.norm(F_x);%TODO

residuals = TEBD_GetResiduals(prob,obj,Norm_Ax_minus_b_ORIG,N_F_x,Norm_Ax_minus_b2);
end

function residuals = GetResiduals2(prob,x,~,F_x,F_u,obj,~,~,~,~)
Ax_minus_b = prob.A*F_u;

Norm_Ax_minus_b_ORIG =  norm(Ax_minus_b);

Norm_Ax_minus_b2 = norm(prob.Chol.S*(prob.Chol.R\(prob.Chol.Rt\(prob.Chol.St*Ax_minus_b))));

N_F_x = prob.norm(F_x);

residuals = TEBD_GetResiduals(prob,obj,Norm_Ax_minus_b_ORIG,N_F_x,Norm_Ax_minus_b2);
end

